"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("./config");
const oas3_chow_chow_1 = require("oas3-chow-chow");
const openapi_ui_1 = require("./openapi-ui");
const jsonfile = require("jsonfile");
const yaml = require("js-yaml");
const fs = require("fs");
const oasValidator = require("oas-validator");
function oas(cfg) {
    const config = config_1.validateConfig(cfg);
    const { compiled, doc } = compileOas(config.file);
    const mw = async (ctx, next) => {
        if (ctx.path === config.endpoint) {
            ctx.body = doc;
            return;
        }
        if (ctx.path === config.uiEndpoint) {
            ctx.body = openapi_ui_1.openapiUI({
                title: doc.info ? doc.info.title : 'openapi UI',
                url: config.endpoint,
            });
            return;
        }
        if (!config.validatePaths.some(path => ctx.path.startsWith(path))) {
            // Skip validation if no path matches
            return await next();
        }
        try {
            compiled.validateRequest(ctx.path, {
                method: ctx.request.method,
                header: ctx.request.header,
                query: ctx.request.query,
                path: ctx.params,
                cookie: ctx.cookies,
                body: ctx.request.body,
            });
        }
        catch (err) {
            console.log(err);
            if (err instanceof oas3_chow_chow_1.ChowError) {
                console.log('instanceof');
                const json = err.toJSON();
                ctx.throw(400, 'Request validation error', Object.assign({ expose: true }, json));
            }
            else {
                console.log('not instanceof');
                throw err;
            }
        }
        await next();
        if (config.validateResponse) {
            try {
                compiled.validateResponse(ctx.path, {
                    method: ctx.method,
                    status: ctx.status,
                    header: ctx.response.header,
                    body: ctx.body
                });
            }
            catch (err) {
                if (err instanceof oas3_chow_chow_1.ChowError) {
                    const json = err.toJSON();
                    ctx.throw(400, 'Response validation error', Object.assign({ expose: true }, json));
                }
                else {
                    throw err;
                }
            }
        }
    };
    return mw;
}
exports.oas = oas;
function compileOas(file) {
    let openApiObject;
    switch (true) {
        case file.endsWith('.json'): {
            openApiObject = jsonfile.readFileSync(file);
            break;
        }
        case file.endsWith('.yaml'):
        case file.endsWith('.yml'): {
            openApiObject = yaml.safeLoad(fs.readFileSync(file, 'utf8'));
            break;
        }
        default:
            throw new Error('Unsupported file format');
    }
    if (!oasValidator.validateSync(openApiObject, {})) {
        throw new Error('Invalid Openapi document');
    }
    return {
        compiled: new oas3_chow_chow_1.default(openApiObject),
        doc: openApiObject,
    };
}
//# sourceMappingURL=index.js.map